@@ -142,7 +142,7 @@ format_message(char **buf, const char *rcode, const char *xcode,
 			len++;
 	}
 
-	if (p > message && p[-1] != '\n')
+	if (p == message || p[-1] != '\n')
 		numlines++;
 	
 	len += numlines * (pfxlen + 2);
