@@ -865,7 +865,6 @@ srv_connect(gacopyz_srv_t srv, char *proto, char *port, char *path)
 				return -1;
 			}
 		}
-#ifdef GACOPYZ_IPV6
 	} else if (strcmp(proto, "inet6") == 0) {
 		int rc;
 		struct addrinfo hints;
@@ -931,7 +930,6 @@ srv_connect(gacopyz_srv_t srv, char *proto, char *port, char *path)
 		memcpy(&addr, res->ai_addr, res->ai_addrlen);
 		socklen = sizeof(addr.sin6);
 		freeaddrinfo(res);
-#endif		
 	} else {
 		gacopyz_io_log(&srv->iod, SMI_LOG_ERR,
 			       _("unsupported protocol: %s"),
@@ -1475,7 +1473,7 @@ format_argv(gacopyz_srv_t srv, char **argv, char **buf, size_t *psize)
 	char **ap;
 	char *p;
 	
-	for (len = 1, ap = argv; *ap; ap++)
+	for (len = 0, ap = argv; *ap; ap++)
 		len += strlen(*ap) + 1;
 	
 	GACOPYZ_ALLOC(srv, *buf = malloc(len));
@@ -1485,7 +1483,6 @@ format_argv(gacopyz_srv_t srv, char **argv, char **buf, size_t *psize)
 		memcpy(p, *ap, len);
 		p += len;
 	}
-	*p = 0;
 	*psize = len;
 }
 
@@ -1534,7 +1531,6 @@ gacopyz_srv_connect(gacopyz_srv_t srv, const char *hostname,
 			break;
 		}
 		
-#ifdef GACOPYZ_IPV6
 		case AF_INET6: {
 			char host[NI_MAXHOST];
 			char service[NI_MAXSERV];
@@ -1561,7 +1557,6 @@ gacopyz_srv_connect(gacopyz_srv_t srv, const char *hostname,
 			break;
 		}
 		
-#endif
 		default:
 			break;
 		}
