@@ -1,5 +1,5 @@
 /* This file is part of Mailfromd.
-   Copyright (C) 2005-2022 Sergey Poznyakoff
+   Copyright (C) 2005-2023 Sergey Poznyakoff
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -447,8 +447,8 @@ dns_query(const char *name, adns_rrtype type, adns_answer **ans_ret)
 
 		/* Follow the CNAME chain. */
 		while (cname_rec.count - 1 <= dns_max_cname_chain) {
-			if (adns_synchronous(state, name, adns_r_cname,
-					     DEFAULT_QFLAGS, &cnans))
+			if ((rc = adns_synchronous(state, name, adns_r_cname,
+						   DEFAULT_QFLAGS, &cnans)))
 				break;
 			if (cnans->status == adns_s_ok) {
 				/*
@@ -480,12 +480,13 @@ dns_query(const char *name, adns_rrtype type, adns_answer **ans_ret)
 				 */
 				free(ans);
 				ans = cnans;
+				break;
 			}
 		}
 		cname_record_list_free(&cname_rec);
 	}
 
-	if (rc == adns_s_ok)
+	if (rc == 0)
 		*ans_ret = ans;
 	else
 		free(ans);
