@@ -189,17 +189,26 @@ flush_deferred_defns (gacopyz_srv_t srv)
 
 /* Expect stuff */
 static char expected_code[4];
+static regex_t expected_re;
+static char *expected_text;
 
 void
-free_expected_code ()
+free_expected_code (void)
 {
   expected_code[0] = 0;
+  if (expected_text)
+    {
+      regfree (&expected_re);
+      free (expected_text);
+      expected_text = NULL;
+    }
 }
 
 int
 set_expected_code (const char *str)
 {
   size_t len = strlen (str);
+
   if (len == 0 || !strchr ("12345", str[0]))
     return 1;
 
@@ -214,6 +223,24 @@ set_expected_code (const char *str)
     len = 3;
   memcpy (expected_code, str, len);
   expected_code[len] = 0;
+
+  return 0;
+}
+
+int
+set_expected_regex (const char *str)
+{
+  int rc = regcomp (&expected_re, str, REG_EXTENDED | REG_NOSUB);
+  if (rc)
+    {
+      char errbuf[512];
+
+      regerror (rc, &expected_re, errbuf, sizeof (errbuf));
+      mu_error ("%s", errbuf);
+      free_expected_code ();
+      return 1;
+    }
+  expected_text = mu_strdup (str);
   return 0;
 }
 
@@ -239,7 +266,27 @@ check_expected_code (char *str)
 	  }
 	exit (1);
       }
-  expected_code[0] = 0;
+
+  str += i;
+  str += strspn (str, " \t");
+  if (expected_text)
+    {
+      if (regexec (&expected_re, str, 0, NULL, 0))
+	{
+	  mu_error (_("expected \"%s\", but got %s"), expected_text, str);
+	
+	  if (gsrv)
+	    {
+	      gacopyz_srv_abort (gsrv);
+	      gacopyz_srv_quit (gsrv);
+	      gacopyz_srv_close (gsrv);
+	      gacopyz_srv_destroy (&gsrv);
+	    }
+	  exit (1);
+	}
+    }
+  
+  free_expected_code ();
 }
 
 
@@ -565,7 +612,6 @@ convert_sender_sockaddr (int argc, char **argv,
     }
       
     case AF_INET6: {
-#ifdef GACOPYZ_IPV6
       struct addrinfo *res;
       int rc;
       
@@ -586,7 +632,6 @@ convert_sender_sockaddr (int argc, char **argv,
       sa = mu_alloc (res->ai_addrlen);
       memcpy (sa, res->ai_addr, res->ai_addrlen);
       freeaddrinfo (res);
-#endif
       break;
     }
       
@@ -748,6 +793,7 @@ portspec_p (const char *str)
 
 pid_t child_pid;
 static char *tmpdir;
+static char *statedir;
 
 static int got_sigchld;
 
@@ -908,25 +954,38 @@ stop_mailfromd (void)
       else
 	mu_error (_("mailfromd terminated with unrecognized status"));
     }
-  
-  rmdir_r (tmpdir);
+
+  if (tmpdir)
+    rmdir_r (tmpdir);
 }
 
+static char dir_prefix[] = "dir:";
+static size_t dir_prefix_len = sizeof (dir_prefix) - 1;
+
+static char unix_prefix[] = "unix://";
+static size_t unix_prefix_len = sizeof (unix_prefix) - 1;
+
 void
 start_mailfromd (int argc, char **argv)
 {
-  tmpdir = mu_strdup ("/tmp/mtasim-XXXXXX");
+  char const *socket_name;
   
-  if (!mkdtemp (tmpdir))
-    {
-      mu_error (_("cannot create temporary directory (%s): %s"),
-		tmpdir, mu_strerror (errno));
-      exit (EX_OSERR);
-    }
   atexit (stop_mailfromd);
 
-  mu_asprintf (&milter_port, "unix://%s/milter", tmpdir);
+  mu_asprintf (&milter_port, "%s%s/milter", unix_prefix, statedir);
+  socket_name = milter_port + unix_prefix_len;
 
+  /*
+   * Presense of socket_name is used as an indicator that mailfromd
+   * has started properly, so make sure no left-over sockets are
+   * here.
+   */
+  if (unlink (socket_name) && errno != ENOENT)
+    {
+      mu_error (_("can't unlink %s: %s"), socket_name, strerror (errno));
+      exit (EX_UNAVAILABLE);
+    }
+  
   signal (SIGCHLD, sig_child);
   
   child_pid = fork ();
@@ -944,7 +1003,7 @@ start_mailfromd (int argc, char **argv)
       int i;
       char *callout_port = NULL;
       
-      mu_asprintf (&callout_port, "unix://%s/callout", tmpdir);
+      mu_asprintf (&callout_port, "unix://%s/callout", statedir);
 
       xargv[0] = "mailfromd";
       for (i = 1; i <= argc; i++)
@@ -957,7 +1016,7 @@ start_mailfromd (int argc, char **argv)
       if (statedir_option)
 	{
 	  xargv[i++] = "--state-directory";
-	  xargv[i++] = tmpdir;
+	  xargv[i++] = statedir;
 	}
       xargv[i] = 0;
 
@@ -978,7 +1037,7 @@ start_mailfromd (int argc, char **argv)
   if (verbose)
     fprintf (stderr, "mailfromd PID: %lu\n", (unsigned long) child_pid);
 
-  while (access (milter_port + 5, F_OK))
+  while (access (socket_name, F_OK))
     {
       struct timeval tv;
       
@@ -1059,7 +1118,24 @@ main (int argc, char **argv)
       gacopyz_set_logger (gacopyz_stderr_log_printer);
       
       if (strcmp (milter_port, "auto") == 0)
-	start_mailfromd (argc, argv);
+	{
+	  tmpdir = mu_strdup ("/tmp/mtasim-XXXXXX");
+	  statedir = tmpdir;
+
+	  if (!mkdtemp (tmpdir))
+	    {
+	      mu_error (_("cannot create temporary directory (%s): %s"),
+			tmpdir, mu_strerror (errno));
+	      exit (EX_OSERR);
+	    }
+	  start_mailfromd (argc, argv);
+	}
+      else if (strlen (milter_port) > dir_prefix_len &&
+	       memcmp (milter_port, dir_prefix, dir_prefix_len) == 0)
+	{
+	  statedir = milter_port + dir_prefix_len;
+	  start_mailfromd (argc, argv);
+	}
 
       if (portspec_p (milter_port))
 	rc = gacopyz_srv_create (&gsrv, "Test", milter_port, gacopyz_log_mask);
@@ -1974,8 +2050,24 @@ struct body_buf
   size_t level;
 };
 
-int
-send_body (int state, char *ptr, size_t len, struct body_buf *buf)
+void
+flush_body_buf (int *state, struct body_buf *buf)
+{
+  char *datareply = NULL;
+  int rc = gacopyz_srv_body (gsrv, (unsigned char*) buf->bufptr, buf->level);
+  buf->level = 0;
+  if (process_data_reply ("cmd", "data", rc, state, &datareply))
+    {
+      if (datareply)
+	{
+	  smtp_send_string (datareply, 0);
+	  free (datareply);
+	}
+    }
+}
+
+void
+send_body (int *state, char *ptr, size_t len, struct body_buf *buf)
 {
   while (len > 0)
     {
@@ -1987,23 +2079,8 @@ send_body (int state, char *ptr, size_t len, struct body_buf *buf)
       len -= s;
       buf->level += s;
       if (buf->level == max_body_chunk)
-	{
-	  char *datareply = NULL;
-	  int rc = gacopyz_srv_body (gsrv, (unsigned char*) buf->bufptr,
-				     buf->level);
-	  buf->level = 0;
-	  if (process_data_reply ("cmd", "data", rc, &state, &datareply))
-	    {
-	      if (datareply)
-		{
-		  smtp_send_string (datareply, 0);
-		  free (datareply);
-		  datareply = 0;
-		}
-	    }
-	}
+	flush_body_buf (state, buf);
     }
-  return state;
 }
 
 void
@@ -2052,13 +2129,23 @@ smtp (void)
 
       if (state != STATE_HEADERS && state != STATE_DATA)
 	{
-	  mu_rtrim_class (buf, MU_CTYPE_ENDLN);
-	  mu_wordsplit (buf, &ws, wsflags);
+	  int flags = wsflags;
+	  char *bufp = buf;
+	  
+	  mu_rtrim_class (bufp, MU_CTYPE_ENDLN);
+
+	  if (bufp[0] == '\\')
+	    {
+	      bufp++;
+	      flags |= MU_WRDSF_DQUOTE;
+	    }
+
+	  mu_wordsplit (bufp, &ws, flags);
 	  wsflags |= MU_WRDSF_REUSE;
 	  if (ws.ws_wordc == 0)
 	    continue;
       
-	  if (ws.ws_wordv[0][0] == '\\' && milter_port)
+	  if ((flags & MU_WRDSF_DQUOTE) && milter_port)
 	    {
 	      shell (ws.ws_wordc, ws.ws_wordv);
 	      continue;
@@ -2098,7 +2185,7 @@ smtp (void)
 	      datareply = NULL;
 	      if (pool)
 		mu_opool_destroy (&pool);
-	      smtp_reply(250, "2.0.0 Reset state");
+	      smtp_reply (250, "2.0.0 Reset state");
 	      continue;
 	    }
 	}
@@ -2275,8 +2362,10 @@ smtp (void)
 	      {
 		int rc;
 
-		rc = gacopyz_srv_eom (gsrv, (unsigned char *)body_buf.bufptr,
-				      body_buf.level);
+		if (body_buf.level > 0)
+		  flush_body_buf (&state, &body_buf);
+
+		rc = gacopyz_srv_eom (gsrv, NULL, 0);
 		process_data_reply ("cmd", "eom", rc, &state, &datareply);
 		/* FIXME: Clear macro table, except for the entries from
 		   the command line */
@@ -2294,9 +2383,8 @@ smtp (void)
 	  }
 	else if (gsrv)
 	  {
-	    state = send_body (state, buf, len - 1, &body_buf);
-	    if (state == STATE_DATA)
-	      state = send_body (state, "\r\n", 2, &body_buf);
+	    send_body (&state, buf, len - 1, &body_buf);
+	    send_body (&state, "\r\n", 2, &body_buf);
 	  }
 	break;
       }
@@ -2494,14 +2582,14 @@ set_sender_socket (int argc, char **argv)
     {
       int reset = 0;
 
-      if (strcmp (argv[0], "\\S") == 0)
+      if (strcmp (argv[0], "S") == 0)
 	{
 	  ++argv;
 	  --argc;
 	}
       else
 	{
-	  argv[0] += 2;
+	  argv[0]++;
 	  reset = 1;
 	}
       
@@ -2509,7 +2597,7 @@ set_sender_socket (int argc, char **argv)
 				   _sockaddr_error_shell, NULL) == 0)
 	gacopyz_srv_connect (gsrv, sender_hostname, sender_sockaddr);
       if (reset)
-	argv[0] -= 2;
+	argv[0]--;
     }
 }
 
@@ -2518,7 +2606,7 @@ shell_help ()
 {
   static char *hstr[] = {
     " \\Dname=value [name=value...]       Define Sendmail macros",
-    " \\Ecode                             Expect given SMTP reply code",
+    " \\Ecode [regex]                     Expect given SMTP reply code",
     " \\L[name] [name...]                 List macros",
     " \\Uname [name...]                   Undefine Sendmail macros",
     " \\Sfamily hostname address [port]   Define sender socket address",
@@ -2537,7 +2625,7 @@ shell_help ()
 void
 shell (int argc, char **argv)
 {
-  switch (argv[0][1])
+  switch (argv[0][0])
     {
     case '#':
       break;
@@ -2546,8 +2634,8 @@ shell (int argc, char **argv)
     case 'D':
       {
 	int rc = 0;
-	if (argv[0][2])
-	  rc = define_macro (&argv[0][2]);
+	if (argv[0][1])
+	  rc = define_macro (&argv[0][1]);
 	while (rc == 0 && --argc)
 	  rc = define_macro (*++argv);
 
@@ -2558,9 +2646,9 @@ shell (int argc, char **argv)
 
     case 'u':
     case 'U':
-      if (argv[0][2])
+      if (argv[0][1])
 	{
-	  undefine_macro (&argv[0][2]);
+	  undefine_macro (&argv[0][1]);
 	  while (--argc)
 	    undefine_macro (*++argv);
 	}
@@ -2573,9 +2661,9 @@ shell (int argc, char **argv)
 
     case 'l':
     case 'L':
-      if (argv[0][2])
+      if (argv[0][1])
 	{
-	  list_macro (&argv[0][2]);
+	  list_macro (&argv[0][1]);
 	  while (--argc)
 	    list_macro (*++argv);
 	}
@@ -2588,8 +2676,24 @@ shell (int argc, char **argv)
 
     case 'e':
     case 'E':
-      if (set_expected_code (argv[0] + 2))
-	smtp_reply (502, "Invalid SMPT code: %s", argv[0] + 2);
+      if (argc <= 2)
+	{
+	  if (set_expected_code (argv[0] + 1))
+	    {
+	      smtp_reply (502, "Invalid SMPT code: %s", argv[0] + 1);
+	      exit (EX_UNAVAILABLE);
+	    }
+	  if (argc == 2 && set_expected_regex (argv[1]))
+	    {
+	      smtp_reply (502, "Invalid regular expression: %s", argv[1]);
+	      exit (EX_UNAVAILABLE);
+	    }
+	}
+      else
+	{
+	  smtp_reply (502, "Too many arguments");
+	  exit (EX_UNAVAILABLE);
+	}
       break;
 
     case 's':
@@ -2603,7 +2707,7 @@ shell (int argc, char **argv)
       /* \Wseconds */
       {
 	struct timeval tv;
-	if (strtotv (argv[0] + 2, NULL, &tv) == 0)
+	if (strtotv (argv[0] + 1, NULL, &tv) == 0)
 	  {
 	    select (0, NULL, NULL, NULL, &tv);
 	  }
