@@ -1086,7 +1086,7 @@ code_type_result(NODE *node, struct mu_locus_range const **old_locus)
 	
 	code = result_argptr(node->v.ret.code);
 	xcode = result_argptr(node->v.ret.xcode);
-				  
+
 	switch (node->v.ret.stat) {
 	case SMFIS_REJECT:
 		if (code && code->type == node_type_string
@@ -1112,6 +1112,18 @@ code_type_result(NODE *node, struct mu_locus_range const **old_locus)
 	default:
 		break;
 	}		
+
+	if (root_node[smtp_state_action]) {
+		code_result_arg(node->v.ret.message);
+		code_result_arg(xcode);
+		code_result_arg(code);
+		code_op(opcode_push);
+		code_immediate(node->v.ret.stat, long);
+		code_op(opcode_funcall);
+		code_immediate(literal_lookup(PROG_ACTION_NAME)->off, size);
+		action_hook_pc = code_immediate(action_hook_pc, size);
+		*old_locus = NULL;
+	}
 	
 	code_result_arg(node->v.ret.message);
 	code_result_arg(xcode);
@@ -1258,11 +1270,7 @@ mark_type_builtin(NODE *node)
 void
 optimize_type_builtin(NODE *node)
 {
-	size_t i;
-	NODE *p;
-	
-	for (i = 0, p = node->v.builtin.args; p; i++, p = p->next) 
-		optimize(p);
+	optimize(node->v.builtin.args);
 	if (strcmp(node->v.builtin.builtin->name, "interval") == 0) {
 		if (node->v.builtin.args->type == node_type_string) {
 			time_t t;
@@ -2183,7 +2191,7 @@ optimize_type_call(NODE *node)
 	NODE *p;
 	
 	for (p = node->v.call.args; p; p = p->next) 
-		optimize(p);
+		optimize_node(p);
 }
 
 void
@@ -2254,6 +2262,9 @@ code_type_call(NODE *node, struct mu_locus_range const **old_locus)
 	}
 	if (func->rettype != dtype_unspecified && !node->value_ignored)
 		code_op(opcode_pushreg);
+
+	/* Force emission of LOCUS when coding next instruction. */
+	*old_locus = NULL;
 }
 
 
