@@ -231,6 +231,7 @@ xeval(eval_environ_t env, enum smtp_state tag)
 	int rc;
 
 	env_make_frame(env);
+	set_milter_state(env, tag);
 	rc = eval_environment(env, entry_point[tag]);
 	if (rc) 
 		mu_error(_("execution of the filter program was not finished"));
@@ -284,7 +285,6 @@ mlfi_connect(SMFICTX *ctx, char *hostname, _SOCK_ADDR *hostaddr)
 		addrstr = inet_ntoa(hostaddr->sin.sin_addr);
 		break;
 
-#ifdef GACOPYZ_IPV6
 	case PF_INET6: {
 		char host[NI_MAXHOST];
 
@@ -299,7 +299,6 @@ mlfi_connect(SMFICTX *ctx, char *hostname, _SOCK_ADDR *hostaddr)
 			addrstr = "[unresolved]";
 		break;
 	}
-#endif
 		
 	case PF_UNIX:
 		family = MFAM_UNIX;
@@ -672,7 +671,13 @@ child_start()
 	return 0;
 }
 
-enum smtp_state first_used_state = smtp_state_none;
+/*
+ * Index of the first milter state for which a "prog" is registered.
+ * "First" stands here for the state which will be entered first in
+ * the normal message processing sequence.  It is used to register the
+ * "i" macro to use in that state.
+ */
+static enum smtp_state first_used_state = smtp_state_none;
 
 sfsistat
 mlfi_negotiate(SMFICTX *ctx,
@@ -696,7 +701,7 @@ mlfi_negotiate(SMFICTX *ctx,
 	    && first_used_state <= smtp_state_last) {
 		mu_debug(MF_SOURCE_ENGINE, MU_DEBUG_TRACE6, 
 			 ("Registering \"i\" macro for state %s",
-			  state_to_string(first_used_state)));
+			  xstate_to_string(first_used_state)));
 		register_macro(first_used_state, "i"); 
 	}
 	
@@ -791,7 +796,7 @@ void
 milter_enable_state(enum smtp_state state)
 {
 	if (state >= smtp_state_first && state <= smtp_state_last
-	    && state < first_used_state)
+	    && (first_used_state == smtp_state_none || state < first_used_state))
 		first_used_state = state;
 	
 	switch (state) {
