@@ -40,27 +40,33 @@
 #include <mflib/exceptions.h>
 
 /* SMTP states */
+#include <mflib/milter.h>
 enum smtp_state {
-	smtp_state_none,
-	smtp_state_startup,
-	smtp_state_shutdown,
-	smtp_state_begin,
-	smtp_state_end,
-	smtp_state_connect,
-	smtp_state_first=smtp_state_connect,
-	smtp_state_helo,
-	smtp_state_envfrom,
-	smtp_state_envrcpt,
-	smtp_state_data,
-	smtp_state_header,
-	smtp_state_eoh,
-	smtp_state_body,
-	smtp_state_eom,
-	smtp_state_last=smtp_state_eom,
+	smtp_state_none = _MFL_milter_state_none,
+	smtp_state_startup = _MFL_milter_state_startup,
+	smtp_state_shutdown = _MFL_milter_state_shutdown,
+	smtp_state_begin = _MFL_milter_state_begin,
+	smtp_state_end = _MFL_milter_state_end,
+	smtp_state_connect = _MFL_milter_state_connect,
+	smtp_state_helo = _MFL_milter_state_helo,
+	smtp_state_envfrom = _MFL_milter_state_envfrom,
+	smtp_state_envrcpt = _MFL_milter_state_envrcpt,
+	smtp_state_data = _MFL_milter_state_data,
+	smtp_state_header = _MFL_milter_state_header,
+	smtp_state_eoh = _MFL_milter_state_eoh,
+	smtp_state_body = _MFL_milter_state_body,
+	smtp_state_eom = _MFL_milter_state_eom,
 
-	smtp_state_count
+	smtp_state_action = _MFL_milter_action,
+
+	smtp_state_count,
 };
 
+#define smtp_state_first smtp_state_connect
+#define smtp_state_last smtp_state_eom
+
+#define PROG_ACTION_NAME "prog action"
+
 #define MFAM_STDIO 0
 #define MFAM_UNIX  1
 #define MFAM_INET  2
@@ -370,6 +376,7 @@ struct loop_node {
 struct argx_node {
 	int nargs;
 	NODE *node;
+	data_type_t dtype;
 };
 
 struct sedcomp_node {
@@ -764,6 +771,7 @@ void parse_pragma(const char *text);
 struct mu_locus_range const *get_locus(void);
 const char *msgmod_opcode_str(enum msgmod_opcode opcode);
 const char *sfsistat_str(sfsistat s);
+int sfsistat_code(char const *stat);
 const char *mf_exception_str(unsigned ex);
 
 mf_exception  mf_status_to_exception(mf_status s);
@@ -771,6 +779,7 @@ mf_exception  mf_status_to_exception(mf_status s);
 const char *storage_class_str(storage_class_t sc);
 enum smtp_state string_to_state(const char *name);
 const char *state_to_string(enum smtp_state state);
+const char *xstate_to_string(enum smtp_state state);
 void parse_error(const char *fmt, ...) MU_PRINTFLIKE(1,2);
 void parse_error_locus(struct mu_locus_range const *loc, const char *fmt, ...)
 	               MU_PRINTFLIKE(2,3);
@@ -812,6 +821,8 @@ void variable_check_initialized(struct variable *var,
 /* Entry points for each SMTP state */
 extern prog_counter_t entry_point[smtp_state_count];
 
+void set_milter_state(eval_environ_t env, enum smtp_state state);
+
 /* Functions for accessing %rcpt_count */
 unsigned long get_rcpt_count(eval_environ_t env);
 void clear_rcpt_count(eval_environ_t env);
