@@ -710,7 +710,7 @@ runtime_warning(eval_environ_t env, const char *fmt, ...)
 		       _("RUNTIME WARNING near %s:%u: "),
 		       ENV_LOC_FILE(env), ENV_LOC_LINE(env));
 	va_start(ap, fmt);
-	mu_diag_cont_printf(fmt, ap);
+	mu_diag_cont_vprintf(fmt, ap);
 	va_end(ap);
 	mu_diag_cont_printf("\n");
 }
@@ -1931,7 +1931,7 @@ instr_result(eval_environ_t env)
 			   SP(code),
 			   SP(xcode),
 			   SP(message));
-	
+
 	trace("%s%s:%u: %s %s %s %s",
 	      mailfromd_msgid(env->ctx),
 	      ENV_LOC_FILE(env), ENV_LOC_LINE(env),
@@ -1950,6 +1950,7 @@ instr_result(eval_environ_t env)
 				       "modification commands to be ignored; "
 				       "call mmq_purge() prior to `accept', "
 				       "to suppress this warning"));
+
 	env->status = status;
 	env->setreply(env->data, code, xcode, message);
 	advance_pc(env, 1);
