@@ -120,13 +120,22 @@ swapd(char *a, char *b, size_t len)
 }
 
 static char *
-scanback(char *s, int c, size_t n)
+scanforw(char *s, char const *delim)
+{
+	for (; *s; s++)
+		if (strchr(delim, *s))
+			return s;
+	return NULL;
+}
+
+static char *
+scanback(char *s, char const *delim, size_t n)
 {
 	s += n;
 	while (n > 0) {
 		n--;
 		s--;
-		if (*s == c)
+		if (strchr(delim, *s))
 			return s;
 	}
 	return NULL;
@@ -134,7 +143,7 @@ scanback(char *s, int c, size_t n)
 
 /* Reverse the order of parts of BUF, delimited by DELIM */
 void
-spf_reverse(char *buf, size_t len, int delim)
+spf_reverse(char *buf, size_t len, char const *delim)
 {
 	char *lp, *rp;
 	size_t llen, rlen;
@@ -144,7 +153,7 @@ spf_reverse(char *buf, size_t len, int delim)
 	size_t bufsize = strlen(buf);
 	
 	do {
-		lp = memchr(buf, delim, bufsize);
+		lp = scanforw(buf, delim);
 		if (!lp)
 			break;
 		llen = lp - buf;
@@ -188,7 +197,7 @@ spf_reverse(char *buf, size_t len, int delim)
    NPARTS right-hand parts of them. If NPARTS is greater than the
    actual number of parts, then do nothing */
 void
-spf_truncate(char *buffer, int delim, unsigned nparts)
+spf_truncate(char *buffer, char *delim, unsigned nparts)
 {
 	char *p;
 	size_t len;
@@ -211,15 +220,17 @@ spf_truncate(char *buffer, int delim, unsigned nparts)
 
 /* Replace with dots all occurrences of DELIM in first LEN bytes of BUF */
 void
-spf_repl(char *buf, size_t len, int delim)
+spf_repl(char *buf, size_t len, char const *delim)
 {
 	while (len--) {
-		if (*buf == delim)
+		if (strchr(delim, *buf))
 			*buf = '.';
 		buf++;
 	}
 }
 
+#define DELIMITERS ".-+,/_="
+
 /* Perform an RFC 4408 transformation.
    DAT - current spf_data
    X   - the substituted value
@@ -233,7 +244,8 @@ spf_transform(struct spf_data *dat, const char *x, size_t l, const char **pptr)
 	const char *p = *pptr;
 	int nparts = 0;
 	int reverse = 0;
-	int delim = '.';
+	char delim[sizeof(DELIMITERS)] = ".";
+	int dlen;
 	
 	while (mu_isdigit(*p))
 		nparts = nparts * 10 + *p++ - '0';
@@ -242,8 +254,14 @@ spf_transform(struct spf_data *dat, const char *x, size_t l, const char **pptr)
 		p++;
 		reverse = 1;
 	}
-	if (*p && strchr(".-+,/_=:", *p))
-		delim = *p++;
+
+	if (*p && (dlen = strspn(p, ".-+,/_=")) > 0) {
+		if (dlen > sizeof(delim)-1)
+			dlen = sizeof(delim)-1;
+		memcpy(delim, p, dlen);
+		delim[dlen] = 0;
+		p += dlen;
+	}
 	if (*p != '}')
 		return 1;
 	*pptr = p + 1;
@@ -261,8 +279,7 @@ spf_transform(struct spf_data *dat, const char *x, size_t l, const char **pptr)
 		spf_truncate(dat->buffer, delim, nparts);
 		l = strlen(dat->buffer);
 	}
-	if (delim != '.')
-		spf_repl(dat->buffer, l, delim);
+	spf_repl(dat->buffer, l, delim);
 
 	mu_opool_append(dat->tmpool, dat->buffer, l);
 
@@ -1190,6 +1207,23 @@ spf_test_record(const char *rec, spf_query_t *q, spf_answer_t *a)
 	return result;
 }	
 
+char *
+spf_test_macro_expand(const char *input, spf_query_t *q, int allow_exp)
+{
+	struct spf_data dat;
+	char *res;
+
+	if (spf_data_init(&dat, q, NULL, 0))
+		return NULL;
+	if (spf_macro_expand(input, &dat, allow_exp)) {
+		spf_data_free(&dat);
+		return NULL;
+	}
+	res = mu_strdup(spf_data_ptr(&dat));
+	spf_data_free(&dat);
+	return res;
+}
+
 /* Implementation of check_host() function. RFC 4408, 4. */
 spf_result
 spf_check_host_internal(spf_query_t *q, spf_answer_t *a, size_t loopno)
@@ -1217,8 +1251,6 @@ spf_check_host_internal(spf_query_t *q, spf_answer_t *a, size_t loopno)
 	
 	if (a)
 		memset(a, 0, sizeof *a);
-	if (strlen(q->domain) > 63) 
-		SPF_RETURN(spf_none, "domain too long");
 	
 	switch (spf_lookup(q->domain, &spf_rec)) {
 	case dns_success:
